//////////////////////////////////////////////////////////////////////////////////////////
//                                                                                      //
//      Filename                :               User.h                                  //
//                                                                                      //
//      Author                  :               Raman Dhillon                           //
//                                                                                      //
//      Version                 :               1.0                                     //
//                                                                                      //
//      Location                :               Pervasive Communication Lab             //
//                                              University of Colorado at Boulder       //
//                                                                                      //
//      Change Log                                                                      //
//      --------------------------------------------------------------------------      //
//      Change          Author          Date            Reason                          //
//                                                                                      //
//      --------------------------------------------------------------------------      //
//                                                                                      //
//////////////////////////////////////////////////////////////////////////////////////////


#ifndef _USER_H
#define _USER_H

#include <iostream>
#include <iomanip>

#include <vector>
#include "TrafficInfo.h"
#include "UserThread.h"
#include "UserStats.h"
#include "POSIXThread.h"
#include "Barrier.h"


using namespace std;


class User
{
  vector <UserThread *> userThreads;

  TrafficInfo* trafficInfo;
  UserStats* userStats;	
  UserStats* allUserStats;	

  POSIXThread* userThread;
  int userID;

 public:

  User(int userid, Barrier* barr, TrafficInfo* trafInfo, UserStats* finalStats)
  : userID(userid), trafficInfo(trafInfo), allUserStats(finalStats)
  {
	userStats = new UserStats(userid);
	userThread = new UserThread(userid, barr, trafficInfo, userStats);

  }
  
  ~User() 
  { 
   	userThread->join();

	int fileSizeUnits = trafficInfo->getFileSizeUnits();

	string units;

	double bytesRecv = userStats->getBytesRecv();

        switch(fileSizeUnits)
        {
         	case KBYTES: 
				units = "kB"; 
				bytesRecv = bytesRecv / 1000;
				break;

                case MBYTES: 
			     	units = "MB"; 
			     	bytesRecv = bytesRecv / 1000000;
			     	break;

                case GBYTES: 
				units = "GB"; 
				bytesRecv = bytesRecv / 1000000000;
				break;
        }	
	
	cout << "\n#" << setw(3) << userStats->getUserId() << "   " << setw(5);

	cout << userStats->getNoOfReqSent() << "   " << setw(5);
	cout << userStats->getNoOfRespRecv() << "   " << setw(7);

	cout << bytesRecv << " " << units << "   " << setw(10);
	cout << userStats->getDownloadRate() << " kbps   " << setw(10);


	
	cout << userStats->getMinFileSize() << " " << units << "   " << setw(10);
	cout << userStats->getMaxFileSize() << " " << units << "   " << setw(10);
	cout << userStats->getMinInterArrTime() << " s   " << setw(10);
	cout << userStats->getMaxInterArrTime() << " s" ;

	allUserStats->updateBytesRecv((unsigned int)userStats->getBytesRecv());

	if(userThread)
	{
		delete userThread;
		userThread = NULL;
	}

	if(userStats)
	{
		delete userStats;
		userStats = NULL;
	}

  }

  void simulateUser()
  {
   	userThread->start(true);
  }

  void terminateSimulation()
  {
	userThread->collectFinalStats();

	userThread->terminate();
  }


};

#endif
