//////////////////////////////////////////////////////////////////////////////////////////
//                                                                                      //
//      Filename                :               UserStats.h                             //
//                                                                                      //
//      Author                  :               Raman Dhillon                           //
//                                                                                      //
//      Version                 :               1.0                                     //
//                                                                                      //
//      Location                :               Pervasive Communication Lab             //
//                                              University of Colorado at Boulder       //
//                                                                                      //
//      Change Log                                                                      //
//      --------------------------------------------------------------------------      //
//      Change          Author          Date            Reason                          //
//                                                                                      //
//      --------------------------------------------------------------------------      //
//                                                                                      //
//////////////////////////////////////////////////////////////////////////////////////////


#ifndef _USER_STATS_H
#define _USER_STATS_H

#include <iostream>
#include <sys/time.h>

using namespace std;

class UserStats
{
   int userID;
   
   unsigned int noOfReqSent;
   unsigned int noOfRespRecv;

   double bytesSent;
   double bytesRecv;

   double meanDataRate;
  
   struct timeval *simStartTime;
   struct timeval *simEndTime;


   double minInterArrTime;
   double maxInterArrTime;

   double minFileSize;
   double maxFileSize;


   public:

	UserStats(int userid): userID(userid) 
	{
		noOfReqSent = 0;
		noOfRespRecv = 0;

		bytesSent = 0;
		bytesRecv = 0;

		meanDataRate = 0.0;

		minInterArrTime = maxInterArrTime = 0.0;
		minFileSize = maxFileSize = 0.0;
	

		simStartTime = new timeval;
		simEndTime = new timeval;
		
	}
	~UserStats() 
	{
		delete simStartTime;
		delete simEndTime;

	}

	int getUserId() { return userID;}

	void incrNoOfReqSent() { noOfReqSent++;}
	int getNoOfReqSent() { return noOfReqSent;}

 	void incrNoOfRespRecv() { noOfRespRecv++;}
	int getNoOfRespRecv() { return noOfRespRecv;}

	void updateBytesSent(int noOfBytes) { bytesSent += noOfBytes;}
	double getBytesSent() { return bytesSent;}

	void updateBytesRecv(int noOfBytes) { bytesRecv += noOfBytes;}
	double getBytesRecv() { return bytesRecv;}

	double getDownloadRate() 
	{

		double duration = ((simEndTime->tv_sec - simStartTime->tv_sec) * 1000 + (simEndTime->tv_usec - simStartTime->tv_usec)/1000)/1000;
	
		double rate = (bytesRecv*8) /(1000* duration );
		 
		return rate;

	}
	double getUploadRate() { return (bytesSent*8) / (1000* (simEndTime - simStartTime));} 
	
	double getTestDuration() 
	{
		double duration = ((simEndTime->tv_sec - simStartTime->tv_sec) * 1000 + (simEndTime->tv_usec - simStartTime->tv_usec)/1000)/1000;
		
		return duration;
	}

	void recordStartTime() { gettimeofday(simStartTime,NULL);}
	void recordEndTime() { gettimeofday(simEndTime, NULL);}

	int setInterArrTime(double interArr)
	{
	
		if(interArr > minInterArrTime && interArr < maxInterArrTime)

			return 0;
	
		if(maxInterArrTime != 0 && interArr > maxInterArrTime) 
		{
			maxInterArrTime = interArr;
			return 0;
		}

		if(minInterArrTime != 0 && interArr < minInterArrTime)
		{
			minInterArrTime = interArr;
			return 0;
		}

		minInterArrTime = maxInterArrTime = interArr;

		return 0;
	}

	int setFileSize(double fileSize)
	{

		if(fileSize > minFileSize && fileSize < maxFileSize)
	
			return 0;


		if(maxFileSize != 0 && fileSize > maxFileSize)
		{
			maxFileSize = fileSize;
			return 0;
		}

		if(minFileSize != 0 && fileSize < minFileSize)
		{
			minFileSize = fileSize;
			return 0;
		}

		minFileSize = maxFileSize = fileSize;

	}

	double getMinInterArrTime()	{ return minInterArrTime; }
	double getMaxInterArrTime() 	{ return maxInterArrTime; }

	double getMinFileSize()		{ return minFileSize; }
	double getMaxFileSize()		{ return maxFileSize; }



};

#endif
