//////////////////////////////////////////////////////////////////////////////////////////
//                                                                                      //
//      Filename                :               UserThread.h                            //
//                                                                                      //
//      Author                  :               Raman Dhillon                           //
//                                                                                      //
//      Version                 :               1.0                                     //
//                                                                                      //
//      Location                :               Pervasive Communication Lab             //
//                                              University of Colorado at Boulder       //
//                                                                                      //
//      Change Log                                                                      //
//      --------------------------------------------------------------------------      //
//      Change          Author          Date            Reason                          //
//                                                                                      //
//      --------------------------------------------------------------------------      //
//                                                                                      //
//////////////////////////////////////////////////////////////////////////////////////////


#ifndef _USER_THREAD_H
#define _USER_THREAD_H

#include <iostream>
#include "POSIXThread.h"
#include "Barrier.h"
#include "ParetoDist.h"
#include "LogNormalDist.h"
#include "HTTPHandler.h"
#include "UserStats.h"
#include "TrafficInfo.h"

using namespace std;

class UserThread : public POSIXThread
{
   HTTPHandler* http;
   Barrier* barr;
   ParetoDist* intArrvDist;
   LogNormalDist* fileSizeDist;
   TrafficInfo* trafficInfo;
   UserStats* userStats;
   string URL;

   static bool simulate;
   static pthread_mutex_t sig_mutex; 	
   int userID;

   public:

    UserThread(int userid, Barrier* b, TrafficInfo* trafInfo, UserStats* userstats);

   ~UserThread();

   static void terminateSimulation();
   static int checkSimulationStatus();

   protected:

    void execute();

    void collectFinalStats();

};


#endif
